#!/usr/bin/perl

use strict;

#This variables must be configured!
use vars qw/$Sisyphus_root @Repos_list/;

#################################################################################################################
#Before running script please:                                                                                  #
#1) Edit root of local repository in this script                                                                #
#2) Create local copy using this script with parameter --create                                                 #
#3) Put i586.rpm to directory /Sisyphus/files/i586/RPMS                                                         #
#        xxx.rpm              /Sisyphus/files/xxx/RPMS                                                          #
#            etc...                                                                                             #
#       src.rpm                /Sisyphus/files/SRPMS                                                            #
#4) Copy files full-plain-index-* from any CD from Delta or Sisyphus                                            #
#   to root of repository and run bunzip2 or gunzip on them. Example: to /Sisyphus                              #
#1) Run this script.                                                                                            #
#################################################################################################################

##########################
#Root of local repository#
##########################

$Sisyphus_root = "";

###########################################
#Which binary repositories you want to use#
###########################################

#@Repos_list = qw/i586 i686 noarch/;
@Repos_list = qw//;





####################################################
####################################################
##                                                ##
##               Do not touch below               ##
##                                                ##
####################################################
####################################################

use RPM::Header;

#Command line parameters
use vars qw/$delete $update $repare $out_file @PARMS/;

#Repository list
use vars qw/%INPUTS $INPUT_FILE/;

#Repository check results
use vars qw/%must_be %must_not_be %broken_name $must_be_name/;

#Work variables
use vars qw/$l $Dir $rpm_info/;

$delete=0;
$update=0;
$repare=0;
undef $out_file;

@PARMS=@ARGV;

print "Repository checker by Epiphanov Sergei\nFor help use -h or --help\n";

if ($Sisyphus_root eq ""){
    print "You must edit your script: value \$Sisyphus_root is not set!\n";
    exit(2);
}

if (scalar(@Repos_list) < 1){
    print "You must edit your script: value \$Repos_list is not set!\n";
    exit(2);
}

#Binary repos
foreach my $i (@Repos_list){
    $INPUTS{"${Sisyphus_root}/full-plain-index-${i}_RPMS"}="${Sisyphus_root}/files/$i/RPMS";
}

#Src repos
$INPUTS{"${Sisyphus_root}/full-plain-index-SRPMS"}="${Sisyphus_root}/files/SRPMS";

while (@PARMS) {
    my $i;
    $i = shift @PARMS;

    if ($i eq "-h" || $i eq "--help"){
	print "This script checks and repares repository tree. It takes files full-plain-index-* from\n";
	print "repository root and scans repository \n";
	print "\nUsage:\n";
	print "   check_repos.pl [-d|--delete|-r|--repare] [-u|--update] [-o outfile|--output outfile]\n";
	print "   check_repos.pl {-c|--create}\n";
	print "   check_repos.pl {-h|--help}\n";
	print "      -d\n";
	print "      --delete         - delete unknown files from repository\n";
	print "      -r\n";
	print "      --repare         - repare broken file names\n";
	print "      -u\n";
	print "      --update         - update repository indexes\n";
	print "      -o outfile\n";
	print "      --output outfile - additionally output all info to file outfile (as tee)\n";
	print "      -c\n";
	print "      --create         - create repository tree\n";
	print "      -h\n";
	print "      --help           - this help\n\n";
	print "      Use -d or -r but not both!\n\n";
	exit;
    }

    if($i eq "-c" || $i eq "--create"){
	mkdir($Sisyphus_root) if ! -e $Sisyphus_root;
	mkdir("$Sisyphus_root/files");

	foreach my $i(@Repos_list){
	    mkdir("$Sisyphus_root/files/$i");
	    mkdir("$Sisyphus_root/$i");
	    mkdir("$Sisyphus_root/$i/base");
	    symlink "../files/$i/RPMS","$Sisyphus_root/$i/RPMS.classic";
	}

	foreach my $i(values %INPUTS){
	    mkdir($i);
	}

        mkdir("$Sisyphus_root/files/SRPMS");
        symlink "files/SRPMS","$Sisyphus_root/SRPMS.classic";
	
#If put this script into root directory of repository
#	my $new_pos;
#	($new_pos)=($0=~/([^\/]+)$/);
#	$new_pos=$Sisyphus_root . "/" . $new_pos;
#	system("cp",$0,$new_pos);

	exit;
    }

    if ($i eq "-d" || $i eq "--delete"){
	$delete=1;
	$repare=0;
	next;
    }

    if ($i eq "-u" || $i eq "--update"){
	$update=1;
	next;
    }

    if ($i eq "-r" || $i eq "--repare"){
	$repare=1;
	$delete=0;
	next;
    }

    if ($i eq "-o" || $i eq "--output"){
	$out_file=shift @PARMS;
	$out_file=~s/[^A-Za-z_=+.-]//g;

	if ($out_file eq ""){
	    printf "You must enter filename for `--output'!\n";
	    exit(1);
	}

	next;
    }

    last if $i eq "--";
    print "Unknown option `$i'!\n";
    exit (1);
}

if (defined $out_file){
    open OUT_FILE,">${out_file}" or die "Can't write to file `$out_file'!\n";
}

print "\n";

foreach $INPUT_FILE (keys %INPUTS) {

    open FILE_LIST,"<$INPUT_FILE" or die "Can't open file `$INPUT_FILE'\n";

# I'll put here all filenames which filenames differ from rpm info
    %broken_name=();

# I'll put here all filenames that must be in repository
    %must_be=();

# I'll put here all filenames that are unknown to list
    %must_not_be=();

    while ($l=<FILE_LIST>) {
	chomp $l;
	$must_be{$l}=1;
    }

    close FILE_LIST;

    $Dir=$INPUTS{$INPUT_FILE};
    opendir LS_LIST,"$Dir" or die;

    while ($l=readdir LS_LIST) {
	chomp $l;
	next if $l eq ".";
	next if $l eq "..";
	$rpm_info = new RPM::Header "$Dir/$l";
	$must_be_name=$rpm_info->{"name"} . "-" . 
		      $rpm_info->{"version"} . "-" .
		      $rpm_info->{"release"} . "." .
		      ($rpm_info->is_source?"src":$rpm_info->{"arch"}) . ".rpm";

	if ($must_be_name ne $l) {
	    $broken_name{$l}=$must_be_name;
	}

	if (exists $must_be{$l}) {
	    delete $must_be{$l};
        } else {
	    $must_not_be{$l}=1;
        }

    }

    closedir LS_LIST;
    ($l) = ($INPUT_FILE =~/.*-([^-]+)$/);
    $l =~ s/_RPMS$//;
    OutInfo("Files not in repository ($l):\n");

    foreach (sort keys %must_be) {
	OutInfo("$Dir/$_\n");
    }

    OutInfo("\nBroken files in repository ($l):\n");

    foreach (sort keys %broken_name) {
	OutInfo("$Dir/$_ must be `$broken_name{$_}'\n");
    }

    OutInfo("\nFiles not in $INPUT_FILE:\n");

    foreach $l (sort keys %must_not_be) {
	OutInfo("$Dir/$l\n");
    }

    if ($delete) {
	OutInfo("Deleting files...\n");

	foreach $l (sort keys %must_not_be) {
	    OutInfo("deleting `$l' ...\n");
	    unlink "$Dir/$l";
	}
    }
    
    if ($repare) {
	OutInfo("Reparing files...\n");

	foreach $l (sort keys %broken_name) {
	    OutInfo("renaming file `$l' to `$broken_name{$l}' ...");
	    Outinfo("NOT done!") unless rename "$Dir/$l","$Dir/$broken_name{$l}";
	    OutInfo("\n"); 
	}

    }
    
    if($update){

	if($INPUT_FILE =~ /-([^-]+)_RPMS$/){
	    OutInfo("\nCreating indexes for $1... \n");
	    system ("genbasedir --topdir=$Sisyphus_root $1 classic");
	}

    }

    OutInfo("Done.\n\n\n");
}

if ($update){
    OutInfo("Updating apt cache...\n");
    system("apt-get update");
    OutInfo("Done\n");
}

system ("/bin/sync");

#Subroutine to output to stdout and file simultaneously
sub OutInfo($){
    my($l)=@_;
    print $l;
    print OUT_FILE $l if defined $out_file;
}
